import os
import argparse

def parse_arguments():
    parser = argparse.ArgumentParser(description='Process some audio files.')
    parser.add_argument('--file_path', type=str, default='test.wav', help='Path to the audio file')
    parser.add_argument('--device', type=str, default='cpu', help='Device to use for processing')
    parser.add_argument('--save', action='store_true', help='Flag to save results to a JSON file')

    args = parser.parse_args()
    return args



import torch
import time
import numpy as np
import librosa
from typing import List, Dict
from whisper_run.pyannote_onnx import PyannoteONNX

class DiarizationPipeline:
    def __init__(self, model_path: str = None, device: str = "cpu", show_progress: bool = False) -> None:
        self.pipeline = PyannoteONNX(show_progress=show_progress)
        self.device = torch.device(device)
        if device != "cpu" and not torch.cuda.is_available():
            raise ValueError("CUDA device specified but CUDA is not available.")
        if device != "cpu":
            raise ValueError("PyannoteONNX models are not typically moved to GPU. They run efficiently on CPU.")

    def run(self, file_path: str) -> List[Dict[str, float]]:
        print(f"Device: {self.device}")
        start_time = time.time()

        # Load audio file
        audio, sample_rate = librosa.load(file_path, sr=self.pipeline.sample_rate, mono=True)
        print(f"Audio shape after loading: {audio.shape}")

        # Process the audio with PyannoteONNX
        diarization_results = list(self.pipeline.itertracks(audio))

        # Convert results to the desired format
        segments = []
        for result in diarization_results:
            segments.append({
                'start': result['start'],
                'end': result['stop'],
                'speaker': f"SPEAKER_{result['speaker']:02d}"
            })

        elapsed_time = time.time() - start_time
        print(f"Diarization completed in {elapsed_time:.2f} seconds")
        return segments


import os
from itertools import permutations
from pathlib import Path
from typing import Union

import librosa
import numpy as np
from tqdm import tqdm

from whisper_run.inference_session import PickableInferenceSession


class PyannoteONNX:
    def __init__(self, show_progress: bool = False):
        # segmentation-3.0 classes:
        #   1. {no speech}
        #   2. {spk1}
        #   3. {spk2}
        #   4. {spk3}
        #   5. {spk1, spk2}
        #   6. {spk1, spk3}
        #   7. {spk2, spk3}
        # only keep the speaker classes
        #   1. {spk1}
        #   2. {spk2}
        #   3. {spk3}
        self.num_classes = 3
        self.sample_rate = 16000
        self.duration = 10 * self.sample_rate
        self.show_progress = show_progress
        onnx_model = f"{os.path.dirname(__file__)}/segmentation-3.0.onnx"
        self.session = PickableInferenceSession(onnx_model)

    @staticmethod
    def sample2frame(x):
        # Conv1d & MaxPool1d & SincNet:
        #   * https://pytorch.org/docs/stable/generated/torch.nn.Conv1d.html
        #   * https://pytorch.org/docs/stable/generated/torch.nn.MaxPool1d.html
        #   * https://github.com/pyannote/pyannote-audio/blob/develop/pyannote/audio/models/blocks/sincnet.py#L50-L71
        #            kernel_size  stride
        # Conv1d             251      10
        # MaxPool1d            3       3
        # Conv1d               5       1
        # MaxPool1d            3       3
        # Conv1d               5       1
        # MaxPool1d            3       3
        # (L_{in} - 721) / 270 = L_{out}
        return (x - 721) // 270

    @staticmethod
    def frame2sample(x):
        return (x * 270) + 721

    @staticmethod
    def sliding_window(waveform, window_size, step_size):
        windows = []
        start = 0
        num_samples = len(waveform)
        while start <= num_samples - window_size:
            windows.append((start, start + window_size))
            yield window_size, waveform[start : start + window_size]
            start += step_size
        # last incomplete window
        if num_samples < window_size or (num_samples - window_size) % step_size > 0:
            last_window = waveform[start:]
            last_window_size = len(last_window)
            if last_window_size < window_size:
                last_window = np.pad(last_window, (0, window_size - last_window_size))
            yield last_window_size, last_window

    @staticmethod
    def reorder(x, y):
        perms = [np.array(perm).T for perm in permutations(y.T)]
        diffs = np.sum(
            np.abs(np.sum(np.array(perms)[:, : x.shape[0], :] - x, axis=1)), axis=1
        )
        return perms[np.argmin(diffs)]

    def __call__(self, x, step=5.0, return_chunk=False):
        step = int(step * self.sample_rate)
        # step: [0.5 * duration, 0.9 * duration]
        step = max(min(step, 0.9 * self.duration), self.duration // 2)
        # overlap: [0.1 * duration, 0.5 * duration]
        overlap = self.sample2frame(self.duration - step)
        overlap_chunk = np.zeros((overlap, self.num_classes))
        windows = list(self.sliding_window(x, self.duration, step))
        if self.show_progress:
            progress_bar = tqdm(
                total=len(windows),
                desc="Pyannote processing",
                unit="frames",
                bar_format="{l_bar}{bar}{r_bar} | {percentage:.2f}%",
            )
        for idx, (window_size, window) in enumerate(windows):
            if self.show_progress:
                progress_bar.update(1)
            ort_outs = self.session.run(None, {"input": window[None, None, :]})[0][0]
            ort_outs = np.exp(ort_outs[:, 1 : self.num_classes + 1])
            # https://herve.niderb.fr/fastpages/2022/10/23/One-speaker-segmentation-model-to-rule-them-all
            # reorder the speakers and aggregate
            ort_outs = self.reorder(overlap_chunk, ort_outs)
            if idx != 0:
                ort_outs[:overlap, :] = (ort_outs[:overlap, :] + overlap_chunk) / 2
            if idx != len(windows) - 1:
                overlap_chunk = ort_outs[-overlap:, :]
                ort_outs = ort_outs[:-overlap, :]
            else:
                # crop
                ort_outs = ort_outs[: self.sample2frame(window_size), :]

            if return_chunk:
                yield ort_outs
            else:
                for out in ort_outs:
                    yield out

    def itertracks(self, wav, onset: float = 0.5, offset: float = 0.5):
        if not isinstance(wav, np.ndarray):
            wav, _ = librosa.load(wav, sr=self.sample_rate, mono=True)

        current_samples = 721
        start = [0] * self.num_classes
        is_active = [False] * self.num_classes
        for speech_probs in self(wav):
            current_samples += 270
            for idx, prob in enumerate(speech_probs):
                if is_active[idx]:
                    if prob < offset:
                        yield {
                            "speaker": idx,
                            "start": round(start[idx] / self.sample_rate, 3),
                            "stop": round(current_samples / self.sample_rate, 3),
                        }
                        is_active[idx] = False
                else:
                    if prob > onset:
                        start[idx] = current_samples
                        is_active[idx] = True
        for idx in range(self.num_classes):
            if is_active[idx]:
                yield {
                    "speaker": idx,
                    "start": round(start[idx] / self.sample_rate, 3),
                    "stop": round(current_samples / self.sample_rate, 3),
                }



import click
import librosa
import matplotlib.pyplot as plt
import numpy as np

from whisper_run.pyannote_onnx import PyannoteONNX


@click.command()
@click.argument("wav_path", type=click.Path(exists=True, file_okay=True))
@click.option("--plot/--no-plot", default=False, help="Plot the vad probabilities")
def main(wav_path: str, plot: bool):
    pyannote = PyannoteONNX()
    for turn in pyannote.itertracks(wav_path):
        print(turn)

    if plot:
        pyannote = PyannoteONNX(show_progress=True)
        wav, sr = librosa.load(wav_path, sr=pyannote.sample_rate)
        outputs = list(pyannote(wav))
        x1 = np.arange(0, len(wav)) / sr
        x2 = [(i * 270 + 721) / sr for i in range(0, len(outputs))]
        plt.plot(x1, wav)
        plt.plot(x2, outputs)
        plt.show()


if __name__ == "__main__":
    main()



import time
from typing import List, Dict
from .audio_converter import AudioConverter
from .diarization_pipeline import DiarizationPipeline
from .transcription_pipeline import TranscriptionPipeline

class AudioProcessor:
    def __init__(self, file_path: str, device: str, model_name: str) -> None:
        self.file_path = file_path
        self.device = device
        self.pyannote_model_name = "./segmentation-3.0.onnx" 
        self.whisper_model_name = model_name

    def process(self) -> Dict[str, List[Dict[str, float]]]:
        total_start_time = time.time()

        self.file_path = AudioConverter.convert_to_wav(self.file_path)

        print("Starting diarization process...")
        diarization_pipeline = DiarizationPipeline(self.pyannote_model_name, self.device)
        diarization_segments = diarization_pipeline.run(self.file_path)
        if not diarization_segments:
            print("No diarization segments found.")

        print("Starting transcription process...")
        transcription_pipeline = TranscriptionPipeline(self.whisper_model_name, self.device)
        transcription_segments = transcription_pipeline.run(self.file_path)

        if diarization_segments:
            for segment in transcription_segments:
                closest_segment = min(
                    diarization_segments,
                    key=lambda x: min(abs(x['start'] - segment['start']), abs(x['end'] - segment['end']))
                )
                segment['speaker'] = closest_segment['speaker'] if closest_segment else None

        final_result = {
            'text': ' '.join([seg['text'] for seg in transcription_segments]),
            'segments': transcription_segments
        }

        total_elapsed_time = time.time() - total_start_time
        print(f"Total processing time: {total_elapsed_time:.2f} seconds")

        return final_result



from .audio_converter import AudioConverter
from .diarization_pipeline import DiarizationPipeline
from .transcription_pipeline import TranscriptionPipeline
from .audio_converter import AudioConverter
from .audio_processor import AudioProcessor
from .config import whisper_models
from .arg_parser import parse_arguments
from .pyannote_onnx import *


from whisper_run import AudioProcessor

def main():
    processor = AudioProcessor(file_path="../../test.wav",
                               device="cpu",
                               model_name="large-v3")
    print(processor.process())

if __name__ == "__main__":
    main()


import onnxruntime as ort


def init_session(model_path):
    opts = ort.SessionOptions()
    opts.inter_op_num_threads = 1
    opts.intra_op_num_threads = 1
    opts.log_severity_level = 3
    sess = ort.InferenceSession(model_path, sess_options=opts)
    return sess


class PickableInferenceSession:
    """
    This is a wrapper to make the current InferenceSession class pickable.
    """

    def __init__(self, model_path):
        self.model_path = model_path
        self.sess = init_session(self.model_path)

    def run(self, *args):
        return self.sess.run(*args)

    def __getstate__(self):
        return {"model_path": self.model_path}

    def __setstate__(self, values):
        self.model_path = values["model_path"]
        self.sess = init_session(self.model_path)



import os
import json
import logging
import inquirer
from whisper_run.audio_processor import AudioProcessor
from whisper_run.config import whisper_models
from whisper_run.arg_parser import parse_arguments  # Use relative import

logging.basicConfig(level=logging.WARNING, format='%(levelname)s: %(message)s')

def main():
    try:
        args = parse_arguments()
    except ValueError as e:
        logging.error(e)
        return

    model_options = {
        f"{model} (fastest)" if model == "distil-whisper/distil-large-v3" else f"{model} (recommended)" if model == "openai/whisper-large-v3" else model: model
        for model in whisper_models
    }

    questions = [
        inquirer.List('model',
                      message="Select a model for audio processing",
                      choices=list(model_options.keys()),
                      ),
    ]
    answers = inquirer.prompt(questions)
    model = model_options[answers['model']]

    file_path = args.file_path
    device = args.device

    processor = AudioProcessor(file_path, device, model)
    results = processor.process()

    results_json = json.dumps(results, indent=4)
    print(results_json)

    if args.save:
        results_dir = "results"
        if not os.path.exists(results_dir):
            os.makedirs(results_dir)

        file_name_without_extension = os.path.splitext(os.path.basename(file_path))[0]
        results_json_path = os.path.join(results_dir, f"{file_name_without_extension}.json")
        with open(results_json_path, "w") as results_json_file:
            json.dump(results, results_json_file, indent=4)
        print(f"Results saved to {results_json_path}")

if __name__ == "__main__":
    main()



from setuptools import find_packages, setup

def read(filename):
    with open(filename, encoding='utf-8') as f:
        return f.read()

def load_requirements(file_path: str):
    with open(file_path, 'r', encoding='utf-8') as f:
        return [line.strip() for line in f if line.strip()]

setup(
    name='whisper-run',
    version='1.2.1',
    author='Görkem Karamolla',
    author_email='gorkemkaramolla@gmail.com',
    description='Whisper with speaker diarization',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    url='https://github.com/gorkemkaramolla/whisper-run',
    package_dir={'': 'src'},
    packages=find_packages(where='src'),
    install_requires=load_requirements('requirements.txt'),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.8',
    include_package_data=True,
    package_data={
        'whisper_run': ['*.onnx'],  # Include ONNX files in the package
    },
    entry_points={
        'console_scripts': [
            'whisper-run=whisper_run.__main__:main'
        ]
    }
)



from pydub import AudioSegment
import os
class AudioConverter:
    @staticmethod
    def convert_to_wav(file_path: str) -> str:
        original = AudioSegment.from_file(file_path)
        wav_path = f"{os.path.splitext(file_path)[0]}.wav"
        original.export(wav_path, format='wav')
        return wav_path




import time
import json
from faster_whisper import WhisperModel
from typing import Dict, List, Any

class TranscriptionPipeline:
    def __init__(self, model_size: str, device: str) -> None:
        self.model = WhisperModel(model_size, device=device, compute_type="int8")

    def run(self, file_path: str) -> List[Dict[str, Any]]:
        start_time = time.time()
        segments, info = self.model.transcribe(file_path, beam_size=5)
        end_time = time.time()

        total_runtime = end_time - start_time
        print(f"Transcription completed in {total_runtime:.2f} seconds")

        transcription_json_start_time = time.time()

        transcription_result = [
            {"start": segment.start, "end": segment.end, "text": segment.text}
            for segment in segments
        ]

        transcription_json_end_time = time.time()
        transcription_json_total = transcription_json_end_time - transcription_json_start_time
        print(f"Transcription JSON preparation {transcription_json_total:.2f} seconds")

        return transcription_result


